#!/bin/bash
# 包含GMT自带脚本文件 其中包含了一些有用的功能 比如获取网格文件的范围
. gmt_shell_functions.sh
#GMT显示平面数据脚本，输入文件为网格文件，没有包含网格化语句因为网格化过程中的情况多样化，建议在其他脚本中个别添加再调用此脚本
# 初始化参数
data='null'
unit='m'
color='rainbow'
range='null'
overwriteRange='null'
overwrite=0
labels=("longitude (degree)" "latitude (degree)")
plot_para=(0 0 0 0)
plotgrad=0
nocpt=0
# 从命令行获取参数
while getopts "hi:r:u:c:a:gv:n" arg
do
	case $arg in
		h)
			printf "simple regional data mapping using GMT script. The template accepts a grid (.nc .grid) file as input and outputs a .png and a .eps file. \
For futher explanations, please look for GMT's manuscripts.\nAuthor: Yi Zhang (zhangyi.cugwuhan@gmail.com)\n" 
			printf "usage: ${0##*/} -i<grid-data> [-r<xmin>/<xmax>/<ymin>/<ymax>] [-u<unit>] [-c<cpt-file>] [-a<x-label>,<y-label>] [-g] [-n]\n"
			printf "%s\t%s\n" "-i" "input grid file"
			printf "%s\t%s\n" "-r" "data range. The template will ditect the input data range automaticly, the use of this option will overwrite the range."
			printf "%s\t%s\n" "-u" "data unit. The default is meter."
			printf "%s\t%s\n" "-c" "color cpt. The default is rainbow. The script will use grd2cpt to recalculate the color scale file, use -n option to disable the use of grd2cpt."
			printf "%s\t%s\n" "-a" "axis labels. The deafults are x (m) and y (m)."
			printf "%s\t%s\n" "-g" "use grdgradient. The default is false."
			printf "%s\t%s\n" "-n" "disabele the use of grd2cpt."
			exit 0;;
		i)
			data=$OPTARG;;
		u)
			unit=$OPTARG;;
		c)
			color=$OPTARG;;
		r)
			overwrite=1
			overwriteRange=$OPTARG;;
		a)
			labels=(${OPTARG//,/ });;
		g)
			plotgrad=1;;
		n)
			nocpt=1;;
		?)
			printf "error: unknow argument\nuse -h option to see help information\n"
			exit 1;;
	esac
done
# 进行必要的参数检查
if [[ $data == "null" ]]; then
	printf "error: no input file name\nuse -h option to see help information\n"
	exit 1
else
	# 初始化临时文件名
	cptfile=user.cpt
	psfile=${data%.*}.ps
	jpgfile=${data%.*}.png
	# 获取网格范围
	if [[ $overwrite == 1 ]]; then
		range=${overwriteRange}
		# 别忘了这里要加上括号 才能初始化数组
		range_sp=(${range//// })
		# 注意bash并不支持浮点运算 所以需要使用bc命令 bc命令是一个计算数学表达式工具 参数是一个数学表达式字符串
		plot_para[0]=$(echo "${range_sp[0]} * 0.5 + ${range_sp[1]} * 0.5"|bc)
		plot_para[1]=$(echo "${range_sp[2]} * 0.5 + ${range_sp[3]} * 0.5"|bc)
		plot_para[2]=$(echo "${plot_para[1]} * 0.5 + ${range_sp[2]} * 0.5"|bc)
		plot_para[3]=$(echo "${plot_para[1]} * 0.5 + ${range_sp[3]} * 0.5"|bc)
	else
		range=$(gmt_get_gridregion ${data})
		range_sp=(${range//// })
		plot_para[0]=$(echo "${range_sp[0]} * 0.5 + ${range_sp[1]} * 0.5"|bc)
		plot_para[1]=$(echo "${range_sp[2]} * 0.5 + ${range_sp[3]} * 0.5"|bc)
		plot_para[2]=$(echo "${plot_para[1]} * 0.5 + ${range_sp[2]} * 0.5"|bc)
		plot_para[3]=$(echo "${plot_para[1]} * 0.5 + ${range_sp[3]} * 0.5"|bc)
	fi
	# 设置绘图参数
	gmt gmtset \
	FONT_ANNOT_PRIMARY=8p,Times-Roman,black \
	MAP_FRAME_PEN=thinnest,black \
	MAP_FRAME_WIDTH=1.5p \
	MAP_TICK_LENGTH_PRIMARY=1.5p/1p \
	MAP_TICK_PEN_PRIMARY=thinnest,black \
	MAP_TITLE_OFFSET=6.5p \
	MAP_GRID_PEN_PRIMARY=thinnest,grey,-- \
	FONT_LABEL=8p,Times-Roman,black \
	MAP_FRAME_AXES=WESnZ \
	COLOR_NAN=black \
	MAP_LABEL_OFFSET=2.5p

	if [[ $nocpt == 1 ]]; then
		gmt grd2cpt ${data} -R${range} -C${color} -Z -D -M > $cptfile
	else
		$cptfile = ${color}
	fi

	if [[ $plotgrad == 1 ]]; then
		gradfile=usergradient.nc
		gmt grdgradient ${data} -G${gradfile} -Nt -A0/45
		gmt grdimage ${data} -R${range} -C${cptfile} -I${gradfile} -Bxag+l"${labels[0]}" -Byag+l"${labels[1]}" -JL${plot_para[0]}/${plot_para[1]}/${plot_para[2]}/${plot_para[3]}/2.5i -K -P > $psfile
	else
		gmt grdimage ${data} -R${range} -C${cptfile} -Bxag+l"${labels[0]}" -Byag+l"${labels[1]}" -JL${plot_para[0]}/${plot_para[1]}/${plot_para[2]}/${plot_para[3]}/2.5i -K -P > $psfile
	fi
	#-C${cptfile}+Uk 使用km（色标单位除1000）
	gmt psscale -Dx0.3i/-0.3i+w1.85i/0.08i+h -C${cptfile} -Bxa -By+l${unit} -O >> $psfile
	gmt psconvert $psfile -A -TEG -E300
	# 删除临时文件 使用linux终端rm命令
	rm $cptfile $psfile gmt.history gmt.conf
	if [[ $plotgrad == 1 ]]; then
		rm $gradfile
	fi
	# 在终端显示图像 此命令需要imgcat.sh脚本和iTerm终端
	#imgcat $jpgfile
	# 打开图片文件 此命令使用MacOS终端open命令
	open $jpgfile
fi